<?php

namespace Drupal\yasm\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\yasm\Services\EntitiesStatisticsInterface;
use Drupal\yasm\Utility\YasmHtml;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * YASM Statistics dashboard controller.
 */
class Dashboard extends ControllerBase {

  /**
   * The current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The entities statitistics service.
   *
   * @var \Drupal\yasm\Services\EntitiesStatisticsInterface
   */
  protected $entitiesStatistics;

  /**
   * Page site content output.
   */
  public function siteContent() {
    return $this->buildContent();
  }

  /**
   * Page my content output.
   */
  public function myContent() {
    $build = $this->buildContent(['uid' => $this->currentUser->id()]);
    // Add user cache context because this can change for every user.
    $build['#cache']['contexts'] = ['user'];

    return $build;
  }

  /**
   * Build content output.
   */
  private function buildContent(array $conditions = []) {
    if (isset($conditions['uid'])) {
      $this->messenger->addMessage($this->t('Statistics filtered with content authored by @user.', [
        '@user' => $this->currentUser->getDisplayName(),
      ]));
      $site_stats = FALSE;
    }
    else {
      $this->messenger->addMessage($this->t('Expensive statistics updates their data 1 time an hour, less expensive statistics updates when the data changes.'));
      $site_stats = TRUE;
    }

    $cards = [];
    // User and site stats.
    if ($this->moduleHandler->moduleExists('file')) {
      $cards[] = $this->buildCard('far fa-file', $this->t('Files'), $this->entitiesStatistics->count('file', $conditions));
    }
    if ($this->moduleHandler->moduleExists('node')) {
      $cards[] = $this->buildCard('far fa-file', $this->t('Nodes'), $this->entitiesStatistics->count('node', $conditions));
    }
    if ($this->moduleHandler->moduleExists('comment')) {
      $cards[] = $this->buildCard('fas fa-comment', $this->t('Comments'), $this->entitiesStatistics->count('comment', $conditions));
    }
    if ($this->moduleHandler->moduleExists('webform')) {
      $cards[] = $this->buildCard('fab fa-wpforms', $this->t('Webforms'), $this->entitiesStatistics->count('webform', $conditions));
      $cards[] = $this->buildCard('fab fa-wpforms', $this->t('Webform submissions'), $this->entitiesStatistics->count('webform_submission', $conditions));
    }

    // Only site stats.
    if ($site_stats) {
      if ($this->moduleHandler->moduleExists('user')) {
        $cards[] = $this->buildCard('fas fa-user', $this->t('Users'), $this->entitiesStatistics->count('user', $conditions));
      }
      if ($this->moduleHandler->moduleExists('taxonomy')) {
        $cards[] = $this->buildCard('fas fa-tags', $this->t('Terms'), $this->entitiesStatistics->count('taxonomy_term', $conditions));
      }
      if ($this->moduleHandler->moduleExists('group')) {
        $cards[] = $this->buildCard('fas fa-users', $this->t('Groups'), $this->entitiesStatistics->count('group', $conditions));
      }
      if ($this->moduleHandler->moduleExists('menu_link_content')) {
        $cards[] = $this->buildCard('fas fa-bars', $this->t('Menus'), $this->entitiesStatistics->count('menu', $conditions));
      }
      if ($this->moduleHandler->moduleExists('block')) {
        $cards[] = $this->buildCard('fas fa-th-large', $this->t('Blocks'), $this->entitiesStatistics->count('block', $conditions));
      }
      if ($this->moduleHandler->moduleExists('views')) {
        $cards[] = $this->buildCard('far fa-list-alt', $this->t('Views'), $this->entitiesStatistics->count('view', $conditions));
      }
    }

    $build = [];
    $build[] = YasmHtml::columns($cards, ['yasm-dashboard']);
    $build[] = [
      '#attached' => [
        'library' => ['yasm/global', 'yasm/fontawesome'],
      ],
      '#cache' => ['max-age' => 3600],
    ];

    return $build;
  }

  /**
   * Build dashboard card.
   */
  private function buildCard($picto, $label, $count) {
    return YasmHtml::picto($picto) . $label . ': ' . $count;
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(AccountInterface $current_user, MessengerInterface $messenger, ModuleHandlerInterface $module_handler, EntitiesStatisticsInterface $entities_statistics) {
    $this->currentUser = $current_user;
    $this->messenger = $messenger;
    $this->moduleHandler = $module_handler;
    $this->entitiesStatistics = $entities_statistics;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('messenger'),
      $container->get('module_handler'),
      $container->get('yasm.entities_statistics')
    );
  }

}
