<?php

namespace Drupal\yasm\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\yasm\Services\EntitiesStatisticsInterface;
use Drupal\yasm\Utility\YasmHtml;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * YASM Statistics site users controller.
 */
class Users extends ControllerBase {

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The entities statitistics service.
   *
   * @var \Drupal\yasm\Services\EntitiesStatisticsInterface
   */
  protected $entitiesStatistics;

  /**
   * A custom access check.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Run access checks for this account.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   The access result.
   */
  public function access(AccountInterface $account) {
    return ($this->moduleHandler->moduleExists('user')) ? AccessResult::allowed() : AccessResult::forbidden();
  }

  /**
   * Site users page output.
   */
  public function siteContent() {
    return $this->buildContent();
  }

  /**
   * Build users page html.
   */
  private function buildContent() {
    $usercount = $this->entitiesStatistics->count('user');
    if ($usercount > 0) {
      // User by status. Exclude user 0 = anonymous.
      $users_status = $this->entitiesStatistics->aggregate('user', ['uid' => 'COUNT'], 'status', [[
        'key'      => 'uid',
        'value'    => 0,
        'operator' => '!=',
      ]]);

      $status_label = [
        1 => $this->t('Active'),
        0 => $this->t('Blocked'),
      ];

      $rows = [];
      if (!empty($users_status)) {
        foreach ($users_status as $users) {
          $rows[] = [
            $status_label[$users['status']],
            $users['uid_count'],
            round($users['uid_count'] * 100 / $usercount, 2) . '%',
          ];
        }
      }

      $table_by_status = YasmHtml::table([
        $this->t('Status'),
        $this->t('Count'),
        $this->t('Percentage'),
      ], $rows);

      // User by role. Exclude user 0 = anonymous.
      $users_roles = $this->entitiesStatistics->aggregate('user', ['uid' => 'COUNT'], 'roles', [[
        'key'      => 'uid',
        'value'    => 0,
        'operator' => '!=',
      ]]);
      $rows = [];
      if (!empty($users_roles)) {
        foreach ($users_roles as $users) {
          if (!empty($users['roles_target_id'])) {
            $role = $this->entityTypeManager->getStorage('user_role')->load($users['roles_target_id'])->label();
          }
          else {
            $role = $this->t('None');
          }

          $rows[] = [
            $role,
            $users['uid_count'],
            round($users['uid_count'] * 100 / $usercount, 2) . '%',
          ];
        }
      }
      $table_by_role = YasmHtml::table([
        $this->t('Role'),
        $this->t('Count'),
        $this->t('Percentage'),
      ], $rows);

      // Count user by acces.
      $access = [
        strtotime('-1 day')  => $this->t('Today'),
        strtotime('-1 week')  => $this->t('Last week'),
        strtotime('-1 month') => $this->t('Last month'),
        strtotime('-1 year') => $this->t('Last year'),
      ];
      $rows = [];
      foreach ($access as $key => $value) {
        $count = $this->entitiesStatistics->count('user', [[
          'key' => 'access',
          'value' => $key,
          'operator' => '>',
        ]]);
        $rows[] = [
          $value,
          $count,
          round($count * 100 / $usercount, 2) . '%',
        ];
      }
      // Count users has never access.
      $never = $this->entitiesStatistics->count('user', ['access' => 0]);
      $rows[] = [
        $this->t('Never'),
        $never,
        round($never * 100 / $usercount, 2) . '%',
      ];
      $table_by_access = YasmHtml::table([
        $this->t('Access'),
        $this->t('Count'),
        $this->t('Percentage'),
      ], $rows);

      // Build content output.
      $build = [];
      $build[] = YasmHtml::markup($this->t('There are currently @count users.', ['@count' => $usercount]));

      $cards[] = [
        YasmHtml::title($this->t('Users by status'), 'far fa-user'),
        $table_by_status,
      ];

      $cards[] = [
        YasmHtml::title($this->t('Users by role'), 'far fa-user'),
        $table_by_role,
      ];

      $cards[] = [
        YasmHtml::title($this->t('Users by access'), 'far fa-user'),
        $table_by_access,
      ];

      $build[] = YasmHtml::columns($cards, ['yasm-groups'], 2);

      $build[] = [
        '#attached' => [
          'library' => ['yasm/global', 'yasm/fontawesome', 'yasm/datatables'],
        ],
        '#cache' => [
          'tags' => ['user_list'],
          'max-age' => 3600,
        ],
      ];

      return $build;
    }

    return ['#markup' => $this->t('No data found.')];
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, ModuleHandlerInterface $module_handler, EntitiesStatisticsInterface $entities_statistics) {
    $this->entityTypeManager = $entityTypeManager;
    $this->moduleHandler = $module_handler;
    $this->entitiesStatistics = $entities_statistics;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('yasm.entities_statistics')
    );
  }

}
